@@ -2,6 +2,7 @@
 from gi.repository import Adw, Gtk
 
 from gsecrets.recent_files_menu import RecentFilesMenu
+from gsecrets.recent_manager import RecentManager
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/locked_headerbar.ui")
@@ -11,13 +12,12 @@ class LockedHeaderBar(Adw.Bin):
     title = Gtk.Template.Child()
     split_button = Gtk.Template.Child()
 
-    recents = Gtk.RecentManager.get_default()
-
     def __init__(self):
         super().__init__()
 
+        self.recents = RecentManager()
         self.set_menu()
-        self.recents.connect("changed", self.on_settings_changed)
+        self.recents.changed.connect(self.on_settings_changed)
 
     def set_menu(self):
         menu = RecentFilesMenu()
